from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_Isolation(object):
    def setupUi(self, Isolation):
        if not Isolation.objectName():
            Isolation.setObjectName(u"Isolation")
        Isolation.resize(500, 500)
        self.centralwidget = QWidget(Isolation)
        self.centralwidget.setObjectName(u"centralwidget")
        self.pbtn_list = []
        self.label_title = QLabel(self.centralwidget)
        self.label_title.setObjectName(u"label_title")
        self.label_title.setGeometry(QRect(140, 10, 171, 41))
        font = QFont()
        font.setPointSize(17)
        self.label_title.setFont(font)
        self.label_title.setAlignment(Qt.AlignCenter)
        self.cb_level = QComboBox(self.centralwidget)
        self.cb_level.addItem("")
        self.cb_level.addItem("")
        self.cb_level.addItem("")
        self.cb_level.setObjectName(u"cb_level")
        self.cb_level.setGeometry(QRect(30, 10, 101, 41))
        font1 = QFont()
        font1.setPointSize(15)
        self.cb_level.setFont(font1)
        self.cb_first = QComboBox(self.centralwidget)
        self.cb_first.addItem("")
        self.cb_first.addItem("")
        self.cb_first.setObjectName(u"cb_first")
        self.cb_first.setGeometry(QRect(30, 60, 101, 41))
        self.cb_first.setFont(font1)
        self.pbtn_rule = QPushButton(self.centralwidget)
        self.pbtn_rule.setObjectName(u"pbtn_rule")
        self.pbtn_rule.setGeometry(QRect(310, 10, 181, 41))
        font2 = QFont()
        font2.setPointSize(9)
        self.pbtn_rule.setFont(font2)
        self.label_status = QLabel(self.centralwidget)
        self.label_status.setObjectName(u"label_status")
        self.label_status.setGeometry(QRect(250, 60, 81, 41))
        font3 = QFont()
        font3.setPointSize(13)
        font4 = QFont()
        font4.setPointSize(11)
        self.label_status.setFont(font4)
        self.label_moves = QLabel(self.centralwidget)
        self.label_moves.setObjectName(u"label_moves")
        self.label_moves.setGeometry(QRect(350, 60, 141, 41))
        self.label_moves.setFont(font3)
        self.pbtn_start = QPushButton(self.centralwidget)
        self.pbtn_start.setObjectName(u"pbtn_start")
        self.pbtn_start.setGeometry(QRect(140, 60, 101, 41))
        self.pbtn_start.setFont(font3)
        self.gridLayoutWidget = QWidget(self.centralwidget)
        self.gridLayoutWidget.setObjectName(u"gridLayoutWidget")
        self.gridLayoutWidget.setGeometry(QRect(60, 120, 381, 371))
        self.gridLayout = QGridLayout(self.gridLayoutWidget)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.pbtn_board = QButtonGroup(Isolation)
        self.pbtn_board.setObjectName(u"pbtn_board")
        for i in range(0, 8):
            row = []
            for j in range(0, 8):
                pbtn = QPushButton(self.gridLayoutWidget)
                pbtn.setFont(font)
                self.pbtn_board.addButton(pbtn)
                self.gridLayout.addWidget(pbtn, i, j, 1, 1)
                row.append(pbtn)
            self.pbtn_list.append(row)

        Isolation.setCentralWidget(self.centralwidget)

        self.retranslateUi(Isolation)

        QMetaObject.connectSlotsByName(Isolation)
    # setupUi

    def retranslateUi(self, Isolation):
        for i in range(8):
            for j in range(8):
                if i == 0 and j == 0:
                    self.pbtn_list[i][j].setText(
                        QCoreApplication.translate("Isolation", u"X", None))
                elif i == 7 and j == 7:
                    self.pbtn_list[i][j].setText(
                        QCoreApplication.translate("Isolation", u"O", None))
                else:
                    self.pbtn_list[i][j].setText(
                        QCoreApplication.translate("Isolation", u"", None))

        Isolation.setWindowTitle(
            QCoreApplication.translate("Isolation",
                                       u"\u5b64\u7acb\u6e38\u620f", None))
        self.label_title.setText(
            QCoreApplication.translate("Isolation",
                                       u"\u5b64\u7acb\u6e38\u620f", None))
        self.cb_level.setItemText(0,
                                  QCoreApplication.translate("Isolation",
                                                             u"\u7b80\u5355", None))
        self.cb_level.setItemText(1,
                                  QCoreApplication.translate("Isolation",
                                                             u"\u4e2d\u7b49", None))
        self.cb_level.setItemText(2,
                                  QCoreApplication.translate("Isolation",
                                                             u"\u56f0\u96be", None))

        self.cb_first.setItemText(0,
                                  QCoreApplication.translate("Isolation",
                                                             u"\u5148\u624b", None))
        self.cb_first.setItemText(1,
                                  QCoreApplication.translate("Isolation",
                                                             u"\u540e\u624b", None))

        self.pbtn_rule.setText(
            QCoreApplication.translate("Isolation",
                                       u"\u7b2c\u4e00\u6b21\u73a9\uff1f\u83b7\u53d6\u6e38\u620f\u89c4\u5219",
                                       None))
        self.label_status.setText("")
        self.label_moves.setText("")
        self.pbtn_start.setText(
            QCoreApplication.translate("Isolation",
                                       u"\u5f00\u59cb\u6e38\u620f", None))
        # retranslateUi



class FixPySide2:
    def analyse_code(self, contents: str) -> tuple:
        lines = contents.splitlines(keepends=True)
        # locate pyside_package_dir line index
        pkg_dir = 'os.path.abspath(os.path.dirname(__file__))'
        for index, line in enumerate(lines):
            if pkg_dir in line:
                break

        # return lines, line index, indentation
        return lines, index, ' ' * (len(line.rstrip()) - len(line.strip()))

    def insert_lines(self, contents: str) -> list:
        lines, position, indentation = self.analyse_code(contents)
        new_lines = (
                lines[position] + '\n' +
                indentation + '# add platforms plugin to PATH\n' +
                indentation + 'platforms_dir = ' +
                'os.path.join(pyside_package_dir, "plugins", "platforms")\n' +
                indentation +
                'os.environ["QT_QPA_PLATFORM_PLUGIN_PATH"] = platforms_dir\n')

        # insert lines
        lines[position] = new_lines
        return lines

    def add_plugins_to_PATH(self, file_path):
        # open __init__.py
        with open(file_path, encoding='utf-8') as f:
            contents = f.read()

        # insert lines
        lines = self.insert_lines(contents)

        # save changes to __init__.py
        code = ''.join(lines)
        with open(file_path, 'w', encoding='utf-8') as f:
            f.write(code)

    def start_fix(self):
        """Glitch fixing entry."""
        import PySide2
        PySide2_init_module = PySide2.__file__
        self.add_plugins_to_PATH(PySide2_init_module)


class FixPySide6:
    def analyse_code(self, contents: str) -> tuple:
        lines = contents.splitlines(keepends=True)
        # locate pyside_package_dir line index
        pkg_dir = 'os.path.abspath(os.path.dirname(__file__))'
        for index, line in enumerate(lines):
            if pkg_dir in line:
                break

        # return lines, line index, indentation
        return lines, index, ' ' * (len(line.rstrip()) - len(line.strip()))

    def insert_lines(self, contents: str) -> list:
        lines, position, indentation = self.analyse_code(contents)
        new_lines = (
                lines[position] + '\n' +
                indentation + '# add platforms plugin to PATH\n' +
                indentation + 'platforms_dir = ' +
                'os.path.join(pyside_package_dir, "plugins", "platforms")\n' +
                indentation +
                'os.environ["QT_QPA_PLATFORM_PLUGIN_PATH"] = platforms_dir\n')

        # insert lines
        lines[position] = new_lines
        return lines

    def add_plugins_to_PATH(self, file_path):
        # open __init__.py
        with open(file_path, encoding='utf-8') as f:
            contents = f.read()

        # insert lines
        lines = self.insert_lines(contents)

        # save changes to __init__.py
        code = ''.join(lines)
        with open(file_path, 'w', encoding='utf-8') as f:
            f.write(code)

    def start_fix(self):
        """Glitch fixing entry."""
        import PySide6
        PySide6_init_module = PySide6.__file__
        self.add_plugins_to_PATH(PySide6_init_module)



from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_Calculator(object):
    def setupUi(self, Calculator):
        if not Calculator.objectName():
            Calculator.setObjectName(u"Calculator")
        Calculator.resize(762, 652)
        font = QFont()
        font.setPointSize(11)
        Calculator.setFont(font)
        self.centralwidget = QWidget(Calculator)
        self.centralwidget.setObjectName(u"centralwidget")
        self.pbtn_start = QPushButton(self.centralwidget)
        self.pbtn_start.setObjectName(u"pbtn_start")
        self.pbtn_start.setGeometry(QRect(470, 200, 201, 41))
        font1 = QFont()
        font1.setPointSize(15)
        self.pbtn_start.setFont(font1)
        self.lb_title = QLabel(self.centralwidget)
        self.lb_title.setObjectName(u"lb_title")
        self.lb_title.setGeometry(QRect(111, 50, 561, 101))
        font2 = QFont()
        font2.setPointSize(25)
        self.lb_title.setFont(font2)
        self.lb_title.setAlignment(Qt.AlignCenter)
        self.cb_mode = QComboBox(self.centralwidget)
        self.cb_mode.addItem("")
        self.cb_mode.addItem("")
        self.cb_mode.addItem("")
        self.cb_mode.addItem("")
        self.cb_mode.addItem("")
        self.cb_mode.addItem("")
        self.cb_mode.addItem("")
        self.cb_mode.setObjectName(u"cb_mode")
        self.cb_mode.setGeometry(QRect(100, 200, 231, 41))
        self.cb_mode.setFont(font1)
        self.le_input = QLineEdit(self.centralwidget)
        self.le_input.setObjectName(u"le_input")
        self.le_input.setGeometry(QRect(80, 280, 611, 41))
        font3 = QFont()
        font3.setPointSize(12)
        self.le_input.setFont(font3)
        self.tb_result = QTextBrowser(self.centralwidget)
        self.tb_result.setObjectName(u"tb_result")
        self.tb_result.setGeometry(QRect(85, 380, 611, 241))
        Calculator.setCentralWidget(self.centralwidget)

        self.retranslateUi(Calculator)

        QMetaObject.connectSlotsByName(Calculator)

    # setupUi

    def retranslateUi(self, Calculator):
        Calculator.setWindowTitle(
            QCoreApplication.translate("Calculator",
                                       u"\u6570\u5b66\u8ba1\u7b97\u5668", None))
        self.pbtn_start.setText(
            QCoreApplication.translate("Calculator",
                                       u"\u5f00\u59cb\u8fd0\u7b97", None))
        self.lb_title.setText(
            QCoreApplication.translate("Calculator",
                                       u"\u6570\u5b66\u8ba1\u7b97\u5668", None))
        self.cb_mode.setItemText(
            0, QCoreApplication.translate("Calculator",
                                          u"\u5206\u89e3\u8d28\u56e0\u6570", None))
        self.cb_mode.setItemText(
            1, QCoreApplication.translate("Calculator",
                                          u"\u5206\u5f0f\u901a\u5206", None))
        self.cb_mode.setItemText(
            2, QCoreApplication.translate("Calculator",
                                          u"\u6574\u5f0f\u5927\u9664\u6cd5", None))
        self.cb_mode.setItemText(
            3, QCoreApplication.translate("Calculator",
                                          u"\u591a\u9879\u5f0f\u5c55\u5f00", None))
        self.cb_mode.setItemText(
            4,
            QCoreApplication.translate("Calculator",
                                       u"\u591a\u9879\u5f0f\u56e0\u5f0f\u5206\u89e3",
                                       None))
        self.cb_mode.setItemText(
            5,
            QCoreApplication.translate("Calculator",
                                       u"\u7ed8\u5236\u51fd\u6570\u56fe\u50cf",
                                       None))
        self.cb_mode.setItemText(
            6,
            QCoreApplication.translate("Calculator",
                                       u"\u6c42\u89e3\u65b9\u7a0b\u65b9\u7a0b\u7ec4",
                                       None))

        self.le_input.setText("")
    # retranslateUi



import sys
from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *
from sympy import *
from sympy.abc import*

from NightWindGameLib.Qt.fixQt import FixPySide2
from NightWindGameLib.Qt.UICalculator import Ui_Calculator

fix = FixPySide2()
fix.start_fix()


def get_prime_factor(num):
    number = num
    List = []
    if num >= 2:
        while number > 1:
            for i in range(2, number + 1):
                while number % i == 0:
                    number = number // i
                    List.append(str(i))
    return List


# 多功能数学计算器
class Calculator(QMainWindow, Ui_Calculator):
    def __init__(self):
        super().__init__()
        self.setupUi(self)
        self.expressions = []
        self.expression = None
        self.mode = None
        self.pbtn_start.clicked.connect(self.get_expression)
        self.show()

    # 获取输入的式子
    def get_expression(self):
        try:
            if self.le_input.text() != "":
                self.expressions = str(self.le_input.text()).split(",")
                self.start_calculate()
            else:
                QMessageBox.warning(self, "注意", "你没有输入数学式，请重新输入")
        except TypeError:
            # print(self.expressions)
            if self.le_input.text() != "":
                QMessageBox.warning(self, "注意", "输入有误")

    # 开始运算
    def start_calculate(self):
        QApplication.processEvents()
        self.expression = eval(str(self.expressions[0]))
        self.mode = self.cb_mode.currentText()
        if self.mode == "分解质因数":
            lst_prime = "*".join(get_prime_factor(self.expression))
            string = ""
            for i in lst_prime:
                string += i
            self.tb_result.setText(str(self.expression) + " = " + string)

        elif self.mode == "多项式展开":
            self.tb_result.setText(str(self.expression) + " = " +
                                   str(expand(self.expression)))

        elif self.mode == "多项式因式分解":
            self.tb_result.setText(str(self.expression) + " = " +
                                   str(factor(self.expression)))

        elif self.mode == "分式通分":
            self.tb_result.setText(str(self.expression) + " = " +
                                   str(together(self.expression)))

        elif self.mode == "绘制函数图像":
            pass

        elif self.mode == "求解方程/方程组":
            pass

        elif self.mode == "整式大除法":
            pass

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Return:
            self.get_expression()


# 开始运行
def main():
    app = QApplication(sys.argv)
    window = Calculator()
    sys.exit(app.exec_())


if __name__ == "__main__":
    main()



import sys
import pygame
import random

x = 100
y = 100
speedx = 0.5
speedy = 0.5
landpic = "bg" + str(random.randint(1, 5)) + ".jpg"


# 第1个参数为display,第2个参数为图片名称，第3参数为运动方式(1-3)
def look(screen, picName, way=1):
    global x, y, speedx, speedy, landpic
    picName = picName + ".png"
    myImg = pygame.image.load(picName)
    myImg2 = pygame.transform.scale(myImg, (200, 200))
    # 加载背景
    # bgImg=pygame.image.load(landpic)

    imgRect = myImg.get_rect()

    if way == 1:
        if x >= screen.get_size()[0] - 120 or x <= 0:
            speedx = -speedx
        elif y >= screen.get_size()[1] - 120 or y <= 0:
            speedy = -speedy
        x = x + speedx
        y = y + speedy
    elif way == 2:
        if x >= screen.get_size()[0] - 120 or x <= 0:
            speedx = -speedx
            y = y + 20
        elif y >= screen.get_size()[1] - 120 or y <= 0:
            y = 0
        x = x + speedx
    elif way == 3:
        if x >= screen.get_size()[0] - 120 or x <= 0:
            x = 0
        elif y >= screen.get_size()[1] - 120 or y <= 0:
            speedy = -speedy
            x = x + 20
        y = y + speedy
    # screen.blit(bgImg,(0,0))
    screen.blit(myImg2, (x, y))


def bulid(screen, imgName, x, y):
    img = pygame.image.load(imgName)
    img2 = pygame.transform.scale(img, (200, 200))
    screen.blit(img2, (x, y))


def build(screen, imgName, x, y):
    img = pygame.image.load(imgName)
    img2 = pygame.transform.scale(img, (200, 200))
    screen.blit(img2, (x, y))


def bg(screen):
    img = pygame.image.load("bg.png")
    img2 = pygame.transform.scale(img, (800, 600))
    screen.blit(img2, (0, 0))


def quit():
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            pygame.quit()
            sys.exit()



import os

__all__ = []
__files__ = os.listdir(".")

for file in __files__:
    ext = os.path.splitext(file)[-1]
    name = os.path.splitext(file)[0]

    if ext == ".py" and name != "__init__":
        __all__.append(name)




from NightWindGameLib.Chess.chess import (
    Game, GameState,
    Board, human_player,
    strategic_player,
)


def minimax_search(game: Game, state: GameState):
    player = state.to_move

    def max_value(state):
        # 计算子状态中的最大效用值
        if game.terminal_test(state):
            return game.utility(state=state, player=player)
        v = -float('inf')
        for m in game.moves(state):
            v = max(v, min_value(game.transition(state, m)))
        return v

    def min_value(state):
        # 计算子状态中的最小效用值
        if game.terminal_test(state):
            return game.utility(state=state, player=player)
        v = float('inf')
        for m in game.moves(state):
            v = min(v, max_value(game.transition(state, m)))
        return v

    return max(game.moves(state),
               key=lambda m: min_value(game.transition(state, m)))


class TicTacToe(Game):
    def __init__(self, width=3, height=3, k=3,
                 players=('X', 'O'), to_move='X'):
        self.k = k
        self.players = players
        self.initial = GameState(board=Board(width, height),
                                 to_move=to_move, score=0)
        super().__init__(initial=self.initial, players=self.players)

    def moves(self, state):
        return state.board.blank_squares()

    def transition(self, state, move):
        if move not in self.moves(state):
            return state

        player = state.to_move
        board = state.board.new()
        board[move] = player
        to_move = self.opponent(player)
        score = 0
        if board.k_in_line(self.k, player=player, move=move):
            score = 1 if player == self.initial.to_move else -1

        return GameState(board=board, to_move=to_move, score=score)

    def terminal_test(self, state):
        return not self.moves(state) or state.score != 0

    def utility(self, state, player):
        if player == self.initial.to_move:
            return state.score
        else:
            return -state.score


def ticTacToe():
    minimax_player = strategic_player(minimax_search)
    tic_tac_toe = TicTacToe(players=('X', 'O'), to_move='X')
    first = input("你想先手还是后手？")
    if first == "先手":
        end = tic_tac_toe.play_game(dict(X=human_player, O=minimax_player),
                                    verbose=True)
        result = tic_tac_toe.utility(end, 'X')
        if result == 1:
            print("你赢了！")
        elif result == 0:
            print("平局！")
        elif result == -1:
            print("你输了！")
    else:
        end = tic_tac_toe.play_game(dict(X=minimax_player, O=human_player),
                                    verbose=True)
        result = tic_tac_toe.utility(end, 'O')
        if result == 1:
            print("你赢了！")
        elif result == 0:
            print("平局！")
        elif result == -1:
            print("你输了！")


if __name__ == "__main__":
    ticTacToe()



import json
from arcade import *
from arcade import color

with open("images_morse/morse.json", "r") as f:
    morse = json.load(f)

WIDTH = 300
HEIGHT = 200
TITLE = "发报机"


class Morse(Window):
    def __init__(self, width, height, title):
        super().__init__(width, height, title)
        set_background_color(color.WHITE)
        self.letter = " "
        self.morse = " "
        self.di = load_sound("images_morse/di.wav")
        self.do = load_sound("images_morse/do.wav")

    def on_draw(self):
        start_render()
        x, y = WIDTH / 2, HEIGHT / 2

        draw_text(self.letter, x, y + 15, color.BLUE,
                  font_size=30, font_name=("SimHei", "PingFang"),
                  anchor_x="center", anchor_y="center")
        draw_text(self.morse, x, y - 15, color.RED,
                  font_size=30, font_name=("SimHei", "PingFang"),
                  anchor_x="center", anchor_y="center")

    def on_key_press(self, symbol: int, modifiers: int):
        try:
            code = chr(symbol).upper()
            if code in morse.keys():
                self.letter = code
                self.morse = morse[code]

                for sound in self.morse:
                    if sound == ".":
                        play_sound(self.di)
                        pause(0.1)
                    elif sound == "-":
                        play_sound(self.do)
                        pause(0.3)

        except OverflowError:
            pass


if __name__ == '__main__':
    game = Morse(WIDTH, HEIGHT, TITLE)
    run()



import sys
from PySide2.QtWidgets import*
from PySide2.QtCore import*
from PySide2.QtGui import*

from NightWindGameLib.Qt.fixQt import FixPySide2
fix = FixPySide2()
fix.start_fix()


class Painter(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setup()
        self.show()

    def setup(self):
        self.img = None
        self.material = None
        self.copy_img = None
        self.painter = QPainter()

    def paintEvent(self, event):
        if self.img:
            self.painter.begin(self)
            self.painter.drawPixmap(0, 0, self.img)
            self.painter.end()

    def mousePressEvent(self, event):
        x = event.x()
        y = event.y()
        # print(x, y)
        if self.img and self.material:
            self.painter.begin(self.img)
            w = self.material.width() // 2
            h = self.material.height() // 2
            self.painter.drawPixmap(x - w, y - h, self.material)
            self.painter.end()
            # self.img.save("result.png")
            self.update()

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_1:
            file, _ = QFileDialog.getOpenFileName(caption="选择原图",
                                                  filter="(*.jpg *.png)")
            if file:
                self.img = QPixmap(file)
                self.copy_img = self.img.copy()
                width = self.img.width()
                height = self.img.height()
                self.resize(width, height)

        elif event.key() == Qt.Key_2:
            file, _ = QFileDialog.getOpenFileName(caption="选择素材",
                                                  filter="(*.jpg *.png)")
            if file:
                self.material = QPixmap(file)
                self.setCursor(QCursor(self.material))

        elif event.key() == Qt.Key_3:
            if self.img:
                file, _ = QFileDialog.getSaveFileName(caption="保存结果",
                                                      filter="(*.png *.jpg)")
                if file:
                    self.img.save(file)

        elif event.key() == Qt.Key_4:
            if self.img and self.copy_img:
                self.img = self.copy_img.copy()
                self.update()


def main():
    app = QApplication(sys.argv)
    window = Painter()
    sys.exit(app.exec_())


if __name__ == "__main__":
    main()



from arcade import *
from arcade import key

WIDTH = 700
HEIGHT = 730
TITLE = "加密信件"
MOVE = 1


class Letter(Window):
    def __init__(self, width, height, title):
        super().__init__(width, height, title)
        self.bg = Sprite("images_code/信.png")
        self.bg.center_x = WIDTH // 2
        self.bg.center_y = HEIGHT // 2
        self.tool = Sprite("images_code/密码卡.png")
        self.tool.center_x = self.tool.width // 2
        self.tool.center_y = self.tool.height // 2

    def on_draw(self):
        start_render()
        self.bg.draw()
        self.tool.draw()

    def on_update(self, delta_time: float):
        self.tool.update()

    def on_key_press(self, symbol: int, modifiers: int):
        if symbol == key.UP:
            self.tool.change_y = MOVE
        elif symbol == key.DOWN:
            self.tool.change_y = -MOVE
        elif symbol == key.RIGHT:
            self.tool.change_x = MOVE
        elif symbol == key.LEFT:
            self.tool.change_x = -MOVE

    def on_key_release(self, symbol: int, modifiers: int):
        if symbol == key.UP or symbol == key.DOWN:
            self.tool.change_y = 0
        if symbol == key.LEFT or symbol == key.RIGHT:
            self.tool.change_x = 0


if __name__ == '__main__':
    window = Letter(WIDTH, HEIGHT, TITLE)
    run()



import random
import sys
from PySide2.QtWidgets import *
# 导入界面
from NightWindGameLib.Chess.ui_isolation import Ui_Isolation

# 导入基类和基础方法
from NightWindGameLib.Chess.chess import (
    Game, GameState, Board,
    strategic_player, random_player
)

# 修复Qt
from NightWindGameLib.Qt.fixQt import FixPySide2
fix = FixPySide2()
fix.start_fix()

info = "游戏规则\n双方位于棋盘对角，由X玩家先手下棋，双方可以移动到" \
       "和自己当前格子有公共边或公共点的格子中，某一方走过的地方双方都不允许再次被踏足，" \
       "直到有一方所处格子有公共边或有公共点的格子全部被某一方踏足过，无法移动，游戏结束"


class Isolation(Game):
    def __init__(self, width=8, height=8, players=('X', 'O'), to_move='X'):
        self.players = players
        player_squares = {players[0]: (0, 0),
                          players[1]: (width - 1, height - 1)}
        board = IsolationBoard(width, height)
        for player, square in player_squares.items():
            board[square] = player
        self.initial = IsolationState(board=board,
                                      player_squares=player_squares,
                                      to_move=to_move)
        super().__init__(initial=self.initial, players=self.players)

    def moves(self, state):
        return state.open_moves(state.to_move)

    def transition(self, state, move):
        if move not in self.moves(state):
            return state

        player = state.to_move
        board = state.board.new()
        board.update({move: player, state.player_squares[player]: '*'})
        player_squares = state.player_squares.copy()
        player_squares.update({player: move})
        to_move = self.opponent(player)
        return IsolationState(board=board,
                              player_squares=player_squares,
                              to_move=to_move)

    def utility(self, state, player):
        if player == state.to_move:
            return -1
        else:
            return 1


class IsolationState(GameState):
    def __init__(self, board, player_squares, to_move):
        self.board = board
        self.player_squares = player_squares
        self.to_move = to_move
        super().__init__(board=self.board, to_move=self.to_move)

    def open_moves(self, player):
        return self.board.open_squares(self.player_squares[player])


class IsolationBoard(Board):
    def __init__(self, width, height):
        self.width = width
        self.height = height
        self.squares = {(x, y) for x in range(width)
                        for y in range(height)}
        self.rows = []
        super().__init__(width=self.width, height=self.height)

    def open_squares(self, square):
        open_squares = []
        for delta in ((0, 1), (1, 0), (1, 1), (1, -1)):
            (delta_x, delta_y) = delta
            x, y = square
            x, y = x + delta_x, y + delta_y
            if self.in_board((x, y)) and not self.get((x, y)):
                open_squares.append((x, y))

            x, y = square
            x, y = x - delta_x, y - delta_y
            if self.in_board((x, y)) and not self.get((x, y)):
                open_squares.append((x, y))

        return open_squares

    def in_board(self, square):
        x, y = square
        return 0 <= x < self.width and 0 <= y < self.height

    def __repr__(self):
        for x in range(self.height):
            row = []
            for y in range(self.width):
                row.append(self.get((x, y), " "))
            self.rows.append(row)
        return self.rows


def center_evaluation(game: Isolation, state: IsolationState, player):
    # 占中策略，尽可能往棋盘中间走
    square = state.player_squares[player]
    board = state.board
    center_x, center_y = (board.width + 1) / 2, (board.height + 1) / 2
    return 2 - abs(square[0] - center_x) - abs(square[1] - center_y)


def open_evaluation(game: Isolation, state: IsolationState, player):
    # 开放策略，往自己周围空格子更多的地方走
    own_moves = len(state.open_moves(player))
    opp_moves = len(state.open_moves(game.opponent(player)))
    return own_moves - opp_moves


def mixed_evaluation(game: Isolation, state: IsolationState, player):
    # 综合策略，前期使用占中策略，后期使用开放策略
    if len(state.board.blank_squares()) / len(state.board.squares) >= 0.6:
        return center_evaluation(game, state, player)
    else:
        return open_evaluation(game, state, player)


def minimax_search(game: Game, state: GameState, depth_limit, eval_fn):
    player = state.to_move

    def max_value(state, depth):
        # 计算子状态中的最大效用值
        if game.terminal_test(state):
            return game.utility(state=state, player=player)
        if depth > depth_limit:
            return eval_fn(game, state, player)

        v = -float('inf')
        for m in game.moves(state):
            v = max(v, min_value(game.transition(state, m), depth + 1))
        return v

    def min_value(state, depth):
        # 计算子状态中的最小效用值
        if game.terminal_test(state):
            return game.utility(state=state, player=player)
        if depth > depth_limit:
            return eval_fn(game, state, player)

        v = float('inf')
        for m in game.moves(state):
            v = min(v, max_value(game.transition(state, m), depth + 1))
        return v

    return max(game.moves(state),
               key=lambda m: min_value(game.transition(state, m), 1))


class Main(QMainWindow, Ui_Isolation, Isolation):
    def __init__(self, width=8, height=8, players=('X', 'O'), to_move='X'):
        super().__init__()
        self.setupUi(self)
        self.show()
        self.started = False
        self.pbtn_start.clicked.connect(self.Start)
        self.pbtn_rule.clicked.connect(self.get_rule)
        self.pbtn_board.buttonClicked.connect(self.click)

        self.players = players
        self.player_squares = {players[0]: (0, 0),
                               players[1]: (width - 1, height - 1)}
        self.board = IsolationBoard(width, height)
        for player, square in self.player_squares.items():
            self.board[square] = player
        self.initial = IsolationState(board=self.board,
                                      player_squares=self.player_squares,
                                      to_move=to_move)

        self.center_player = strategic_player(minimax_search,
                                              depth_limit=4,
                                              eval_fn=center_evaluation)
        self.open_player = strategic_player(minimax_search,
                                            depth_limit=4,
                                            eval_fn=open_evaluation)
        self.mixed_player = strategic_player(minimax_search,
                                             depth_limit=4,
                                             eval_fn=mixed_evaluation)
        self.clicked_button = []

    def get_rule(self):
        QMessageBox.about(self, "游戏规则", info)

    def play_game(self, strategies: dict, verbose: bool = False):
        # QApplication.processEvents()
        state = self.initial

        if verbose:
            self.label_moves.setText("初始状态:")

        while not self.terminal_test(state):
            player = state.to_move
            move = strategies[player](self, state)
            state = self.transition(state, move)

        return state

    def human_player(self, game, state: IsolationState):
        QApplication.processEvents()
        open_squares = state.open_moves(state.to_move)
        for i in range(8):
            for j in range(8):
                if (j, i) not in open_squares:
                    self.pbtn_list[i][j].setEnabled(True)
                else:
                    self.pbtn_list[i][j].setCheckable(True)

    def click(self):
        global index

        try:
            clicked = self.pbtn_board.checkedButton()
            clicked.setEnabled(True)
            now = self.player_squares[self.initial.to_move]
            self.clicked_button.append(self.pbtn_list[now[1]][now[0]])
            for i in range(8):
                for j in range(8):
                    btn = self.pbtn_list[i][j]
                    if btn is clicked:
                        clicked.setText(self.initial.to_move)
                        index = (j, i)
                        break

            for i in self.clicked_button:
                i.setEnabled(True)
                for x in self.pbtn_list:
                    for y in x:
                        if i is y:
                            self.pbtn_board.removeButton(i)

            self.label_moves.setText(f"玩家{self.initial.to_move}\n行动:{index}")
            if index in self.board.open_squares(now):
                self.pbtn_list[now[1]][now[0]].setText("*")
                self.pbtn_list[now[1]][now[0]].setEnabled(True)
            self.player_squares[self.initial.to_move] = index

        except AttributeError:
            if not self.started:
                QMessageBox.warning(self, "注意", "你还未开始游戏")

    def Start(self):
        self.started = not self.started

        if self.started:
            level = self.cb_level.currentText()
            first = self.cb_first.currentText()

            if first == "先手":
                self.label_status.setText("你是X玩家")
                if level == "简单":
                    end = self.play_game(
                        dict(X=self.human_player, O=random_player),
                        verbose=True)
                elif level == "中等":
                    end = self.play_game(
                        dict(X=self.human_player,
                             O=random.choice([self.open_player, self.center_player])),
                        verbose=True)
                else:
                    end = self.play_game(
                        dict(X=self.human_player, O=self.mixed_player),
                        verbose=True)

                result = self.utility(state=end, player='X')
                if result == 1:
                    self.label_status.setText("你赢了！")
                elif result == 0:
                    self.label_status.setText("平局！")
                elif result == -1:
                    self.label_status.setText("你输了！")

            else:
                self.label_status.setText("你是O玩家")
                if level == "简单":
                    end = self.play_game(
                        dict(X=self.center_player, O=self.human_player),
                        verbose=True)
                elif level == "中等":
                    end = self.play_game(
                        dict(X=random.choice([self.open_player,
                                              self.center_player]),
                             O=self.human_player),
                        verbose=True)
                else:
                    end = self.play_game(
                        dict(X=self.mixed_player, O=self.human_player), verbose=True)

                result = self.utility(state=end, player='O')
                if result == 1:
                    self.label_status.setText("你赢了！")
                elif result == 0:
                    self.label_status.setText("平局！")
                elif result == -1:
                    self.label_status.setText("你输了！")


if __name__ == "__main__":
    app = QApplication(sys.argv)
    game = Main()
    sys.exit(app.exec_())



import sys
from PySide2.QtWidgets import*
from PySide2.QtCore import*
from PySide2.QtGui import*
from PySide2.QtMultimedia import QSound

from NightWindGameLib.Qt.fixQt import FixPySide2
fix = FixPySide2()
fix.start_fix()


class Tray(QSystemTrayIcon):
    def __init__(self):
        super().__init__()
        self.setup()
        self.show()

    def setup(self):
        self.index = 0
        self.tips = ["我叫编程猫", "很高兴认识你", "很开心能一起学习Python"]
        self.icon = QIcon("images_cat/sit.png")
        self.setIcon(self.icon)
        self.setToolTip("使用鼠标点击我吧")
        self.activated.connect(self.process)
        self.sound1 = QSound("images_cat/sound1.wav")
        self.sound2 = QSound("images_cat/sound2.wav")
        self.icon2 = QIcon("images_cat/stand.png")
        self.timer = QTimer()
        self.icons = [f"images_cat/{i}.png" for i in range(0, 5)]
        self.icon_index = 0
        self.change_icon()
    
    def process(self, key):
        if key == self.Trigger:
            self.sound1.play()
            self.setIcon(self.icon)
        elif key == self.Context or key == self.MiddleClick:
            self.sound2.play()
            self.setIcon(self.icon2)
        self.setToolTip(self.tips[self.index % 3])
        self.index += 1

    def change_icon(self):
        self.setIcon(QIcon(self.icons[self.icon_index % 5]))
        self.icon_index += 1
        self.timer.singleShot(100, self.change_icon)


def main():
    app = QApplication(sys.argv)
    tray = Tray()
    sys.exit(app.exec_())


if __name__ == "__main__":
    main()



from arcade import color
from arcade import *
import random

WIDTH = 800
HEIGHT = 600
TITLE = "Hit The Block"
RECT_WIDTH = 100
RECT_HEIGHT = 10
BLOCK_WIDTH = 80
BLOCK_HEIGHT = 20


# 球拍类
class Rect:
    def __init__(self):
        self.rect_color = color.DARK_BLUE
        self.rect_center_x = WIDTH // 2
        self.rect_center_y = RECT_HEIGHT // 2
        self.rect_move_x = 50

    def drawRect(self):
        draw_rectangle_filled(self.rect_center_x, self.rect_center_y,
                              RECT_WIDTH, RECT_HEIGHT, self.rect_color)

    # 球拍的移动
    def moveRect(self):  # 球拍的y坐标被固定，只考虑是否和窗体左右边缘碰撞
        if self.rect_center_x <= RECT_WIDTH // 2:
            self.rect_center_x = RECT_WIDTH // 2
        if self.rect_center_x >= WIDTH - RECT_WIDTH // 2:
            self.rect_center_x = WIDTH - RECT_WIDTH // 2


# 小球类
class Ball:
    def __init__(self):
        self.ball_color = color.RED
        self.ball_speed_x = random.choice([random.uniform(-5, -3),
                                           random.uniform(3, 5)])
        self.ball_speed_y = random.uniform(3, 5)
        self.ball_radius = 10
        self.ball_center_x = WIDTH // 2
        self.ball_center_y = RECT_HEIGHT + self.ball_radius
        self.rect_x = WIDTH // 2
        self.rect_y = RECT_HEIGHT // 2
        self.over = False

    def drawBall(self):
        draw_circle_filled(self.ball_center_x, self.ball_center_y,
                           self.ball_radius, self.ball_color)

    # 小球的移动
    def moveBall(self):
        # 小球出界
        if self.ball_center_y <= 0:
            self.over = True

        # 小球有没有和窗体左右边缘碰撞
        if self.ball_radius <= self.ball_center_y <= HEIGHT - self.ball_radius:
            if self.ball_center_x <= self.ball_radius or \
                    self.ball_center_x >= WIDTH - self.ball_radius:
                self.ball_speed_x = -self.ball_speed_x

        # 小球有没有和球拍边缘和上边缘碰撞
        elif self.ball_radius <= self.ball_center_x <= WIDTH - self.ball_radius:
            if self.ball_center_y >= HEIGHT - self.ball_radius:
                # 上边缘碰撞
                self.ball_speed_y = -self.ball_speed_y
            elif self.ball_center_y <= self.ball_radius + RECT_HEIGHT:
                # 球拍边缘碰撞
                if self.rect_x <= self.ball_center_x <= self.rect_x + RECT_WIDTH:
                    self.ball_speed_y = -self.ball_speed_y
                else:
                    self.over = True

        self.ball_center_x += self.ball_speed_x
        self.ball_center_y += self.ball_speed_y


# 砖块类
class Brick:
    def __init__(self, center_x, center_y):
        self.color = (139, 126, 102)
        self.center_x = center_x
        self.center_y = center_y
        self.height = BLOCK_HEIGHT
        self.width = BLOCK_WIDTH

    def drawBrick(self):
        draw_rectangle_filled(self.center_x, self.center_y,
                              self.width, self.height, self.color)


# 游戏窗口类
class Game(Window):
    def __init__(self, width, height, title):
        super().__init__(width, height, title)
        self.bg_color = color.SKY_BLUE
        self.ball = Ball()
        self.rect = Rect()
        self.total_time = self.last_time = 0
        self.game_over = False
        self.win = False
        self.score = 0
        set_background_color(self.bg_color)
        self.started = True
        self.block_list = []
        for y in range(570, 209, -50):
            for x in range(40, 761, 144):
                self.block_list.append(Brick(x, y))

    # 绘制界面
    def on_draw(self):
        start_render()
        draw_text(f"Score: {self.score}", 10, 10, color.ORANGE,
                  font_name=("SimHei", "PingFang"), font_size=20)
        self.ball.drawBall()
        self.rect.drawRect()
        for brick in self.block_list:
            brick.drawBrick()

        if self.game_over:  # 失败
            draw_text('Game Over', WIDTH // 2 - 120, HEIGHT // 2,
                      color.RED, font_name=('SimHei', 'PingFang'), font_size=40)

        if self.win:  # 获胜
            draw_text('You Win', WIDTH // 2 - 120, HEIGHT // 2,
                      color.RED, font_name=('SimHei', 'PingFang'), font_size=40)

    # 更新界面
    def on_update(self, delta_time):
        if self.started:
            if self.ball.over:  # 失败
                self.game_over = True

            if len(self.block_list) <= 0:
                self.win = True

            if self.game_over or self.win:
                return

            self.total_time += delta_time
            if int(self.total_time) % 10 == 0 and \
                    int(self.total_time) != int(self.last_time):
                self.ball.ball_speed_x *= 1.25
                self.ball.ball_speed_y *= 1.25
                self.last_time = self.total_time

            self.ball.rect_x = self.rect.rect_center_x - RECT_WIDTH // 2
            self.ball.rect_y = self.rect.rect_center_y + RECT_HEIGHT // 2
            self.ball.moveBall()
            self.collision_with_brick()

    def collision_with_brick(self):
        ball_x, ball_y = self.ball.ball_center_x, self.ball.ball_center_y
        radius = self.ball.ball_radius

        for brick in self.block_list:
            if (brick.center_x - BLOCK_WIDTH // 2 - radius <= ball_x <=
                brick.center_x + BLOCK_WIDTH // 2 + radius) and \
                (abs(brick.center_y + BLOCK_HEIGHT // 2 - ball_y + radius) <= 1 or
                 abs(brick.center_y - BLOCK_HEIGHT // 2 - ball_y - radius) <= 1):
                
                print(1)
                if brick in self.block_list:
                    self.block_list.remove(brick)
                self.score += 1
                self.ball.ball_speed_x *= random.uniform(0.9, 1.1)
                self.ball.ball_speed_y *= random.uniform(0.9, 1.1)
                self.ball.ball_speed_x = -self.ball.ball_speed_x

            elif abs(brick.center_x - ball_x) <= radius + BLOCK_WIDTH // 2 and \
                    abs(brick.center_y - ball_y) <= abs(BLOCK_WIDTH // 2 - radius):

                print(2)
                if brick in self.block_list:
                    self.block_list.remove(brick)

                self.score += 1
                self.ball.ball_speed_x *= random.uniform(0.9, 1.1)
                self.ball.ball_speed_y *= random.uniform(0.9, 1.1)
                self.ball.ball_speed_y = -self.ball.ball_speed_y

    def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
        self.rect.rect_center_x = x
        self.rect.moveRect()


if __name__ == "__main__":
    Game(WIDTH, HEIGHT, TITLE)
    run()



from NightWindGameLib.Chess.chess import (
    Game, GameState,
    Board, human_player,
    strategic_player,
)


def minimax_search(game: Game, state: GameState, depth_limit, eval_fn):
    player = state.to_move

    def max_value(state, depth):
        # 计算子状态中的最大效用值
        if game.terminal_test(state):
            return game.utility(state=state, player=player)
        if depth > depth_limit:
            return eval_fn(game, state, player)

        v = -float('inf')
        for m in game.moves(state):
            v = max(v, min_value(game.transition(state, m), depth + 1))
        return v

    def min_value(state, depth):
        # 计算子状态中的最小效用值
        if game.terminal_test(state):
            return game.utility(state=state, player=player)
        if depth > depth_limit:
            return eval_fn(game, state, player)

        v = float('inf')
        for m in game.moves(state):
            v = min(v, max_value(game.transition(state, m), depth + 1))
        return v

    return max(game.moves(state),
               key=lambda m: min_value(game.transition(state, m), 1))


class TicTacToe(Game):
    def __init__(self, width=3, height=3, k=3,
                 players=('X', 'O'), to_move='X'):
        self.k = k
        self.players = players
        self.initial = GameState(board=Board(width, height),
                                 to_move=to_move, score=0)
        super().__init__(initial=self.initial, players=self.players)

    def moves(self, state):
        return state.board.blank_squares()

    def transition(self, state, move):
        if move not in self.moves(state):
            return state

        player = state.to_move
        board = state.board.new()
        board[move] = player
        to_move = self.opponent(player)
        score = 0
        if board.k_in_line(self.k, player=player, move=move):
            score = 1 if player == self.initial.to_move else -1

        return GameState(board=board, to_move=to_move, score=score)

    def terminal_test(self, state):
        return not self.moves(state) or state.score != 0

    def utility(self, state, player):
        if player == self.initial.to_move:
            return state.score
        else:
            return -state.score


def novice_evaluation(game, state, player):
    import random
    return random.choice([float('inf'), -float('inf')])


def improved_evaluation(game, state: GameState, player):
    value = 0
    board = state.board
    center_x, center_y = (board.width + 1) / 2, (board.height + 1) / 2
    for s, p in board.items():
        distance = abs(s[0] - center_x) + abs(s[1] - center_y)
        if p == player:
            value -= distance
        else:
            value += distance

    return value


class ConnectFour(TicTacToe):
    def __init__(self, width=7, height=6, k=4,
                 players=('X', 'O'), to_move='X'):
        super().__init__(width=width, height=height, k=k,
                         players=players, to_move=to_move)

    def moves(self, state):
        return [(x, y) for (x, y) in state.board.blank_squares()
                if y == state.board.height or (x, y + 1) in state.board]


def connectFour():
    level = input("请选择难度：简单或较难")
    while level != "简单" and level != "较难":
        print("难度输入有误，请重新选择！")
        level = input("请选择难度：简单或较难")

    first = input("你想先手还是后手？")
    while first != "先手" and first != "后手":
        print("先后手输入有误，请重新选择！")
        first = input("你想先手还是后手？")

    novice_player = strategic_player(minimax_search,
                                     depth_limit=4,
                                     eval_fn=novice_evaluation)
    improved_player = strategic_player(minimax_search,
                                       depth_limit=4,
                                       eval_fn=improved_evaluation)

    if first == "先手":
        if level == "简单":
            connect_four = ConnectFour(players=('X', 'O'), to_move='X')
            end = connect_four.play_game(
                dict(X=human_player, O=novice_player), verbose=True)
        else:
            connect_four = ConnectFour(players=('X', 'O'), to_move='X')
            end = connect_four.play_game(
                dict(X=human_player, O=improved_player), verbose=True)

        result = connect_four.utility(end, 'X')
        if result == 1:
            print("你赢了！")
        elif result == 0:
            print("平局！")
        elif result == -1:
            print("你输了！")

    else:
        if level == "简单":
            connect_four = ConnectFour(players=('X', 'O'), to_move='X')
            end = connect_four.play_game(
                dict(X=novice_player, O=human_player), verbose=True)
        else:
            connect_four = ConnectFour(players=('X', 'O'), to_move='X')
            end = connect_four.play_game(
                dict(X=improved_player, O=human_player), verbose=True)

        result = connect_four.utility(end, 'O')
        if result == 1:
            print("你赢了！")
        elif result == 0:
            print("平局！")
        elif result == -1:
            print("你输了！")


if __name__ == "__main__":
    connectFour()
